% Note:
% The code that you are using is free.
% 
% This code is for analysis of lamellar gratings in TE polarization. In 
% this code adaptive spatial resolution technique is employed to provide an
% excellent convergence rate. The code has been written based on [1]. The 
% jump points are also regularized (see [2]).
% 
% [1] A. Khavasi and K. Mehrany, Adaptive Spatial Resolution in Fast,
% Efficient, and Stable Analysis of Metallic Lamellar Gratings at Microwave
% Frequencies, IEEE Transaction on Antenna and Propag., 57, 1115-1127 (2009).
% 
% [2] A. Khavasi, K. Mehrany, Regularization of jump points in applying 
% the adaptive spatial resolution technique, Opt. Commun., 284, 3211-3215 (2011), 
% doi:10.1016/j.optcom.2011.03.037 
% 
% Inputs:
% 
% N: Truncation order
% lambda: Free space wavelength normalized to grating period
% theta: incident angle
% d: Grating thickness normalized to grating period
% n1: Refractive index of incident medium
% n3: Refractive index of transmission medium
% xl: Transition (discontinuity) points in profile of refractive index 
% nl: The refractive index between transitions
% 
% The outputs of this program are diffraction efficiencies which are stored
% in vectors 'DE1' and 'DE3' for reflected and transmitted waves,
% respectively. For example for N = 2 we have:
% DE1 = diffraction efficiency of [2 1 0 -1 -2]'th reflected order
% DE3 = diffraction efficiency of [2 1 0 -1 -2]'th transmitted order
% 
% Example:
% The following figure is one period of a lamellar grating. This grating is
% defined as follows:
% xl = [0 0.2 0.5 0.8 1]; 
% nl = [n22 n21 n22 n21];
% 
%   Incident plane wave
%                   \    |
%                    \   |
%                     \  |   Region 1 (n1)
%                      \ |
%       -    -----------------------------------
%      |     |       |         |        |      |
%   d <      |  n22  |   n21   |   n22  |  n21 |
%      |     |       |         |        |      |
%       -    -----------------------------------
%          x1=0    x2=0.2    x3=0.5   x4=0.8 x5=1
%            <--------------------------------->
%                    Grating Period(=1)         
%                  
%                         Region 3 (n3)
% 

% % % % % 
%Inputs
% % % % % 
N=30;   % Truncation order: Conevergence parameter. N = 30 is large enough for almost all structures
lambda=0.5; % Free space wavelength normalized to grating period
theta=15*pi/180+1e-7; % incident angle
d=1;  % Grating thickness normalized to grating period
n1=1;   % Refractive index of incident medium
n3=1; % Refractive index of transmission medium
% -----------------------------------
% Defining grating region parameteres
% -----------------------------------
% Transition (discontinuity) points in profile of refractive index are
% defined by 'xl'. You must set first and last element of 'xl' to 0 and 1,
% repectively. It means that the first transition occurs at the begining of
% the grating period. 
xl = [0 0.2 0.5 0.8 1]; 

% The refractive index between transitions is determined by 'nl'. 'nl(m)' 
% is the refractive index of the region between xl(m) and xl(m+1).
n21 = 1;
n22 = 0.22-6.71*i; 
nl = [n22 n21 n22 n21];

% -------------------
% Initial computaions
% -------------------
k0=2*pi/lambda;
k1=k0*n1;
k3=k0*n3;
kx=zeros(2*N+1,1);
kx(1:2*N+1)=2*pi*(n1*sin(theta)/lambda-(-N:N));
kz1(1:2*N+1)=sqrt(k1^2-kx(1:2*N+1).^2);
kz3(1:2*N+1)=sqrt(k3^2-kx(1:2*N+1).^2);

for j=1:2*N+1
    if -real(kz1(j))>1e-12
        kz1(j)=-kz1(j);
    end
    if imag(kz1(j))>1e-12
        kz1(j)=-kz1(j);
    end
    
    if -real(kz3(j))>1e-12
        kz3(j)=-kz3(j);
    end
    if imag(kz3(j))>1e-12
        kz3(j)=-kz3(j);
    end
end

Kx=diag(kx);
Kz1=diag(kz1);
Kz3=diag(kz3);
I=eye(2*N+1);
Zero=zeros(2*N+1);
% -------------------

% --------------
% New coordinate
% --------------
G=0.001;

% -------------------------------------------------------------------------
% In this part, transition points in new coordinate, 'ul's, are chosen in
% an efficient way based on the arqument in Ref. 2
% -------------------------------------------------------------------------
dx = zeros(1,length(xl)-1);
du=dx;
for j=1:(length(xl)-1)
    dx(j)=(xl(j+1)-xl(j))^(1/3);
end
ul=0*xl;
for j=1:(length(xl)-1)
    du(j)=dx(j)/sum(dx);
    ul(j+1)=ul(j)+du(j);
end

if G==1
    ul=xl;
end
epsl=nl.^2;

ulen=10*N;
u=(0:(ulen))/(ulen+1);
x=zeros(1,ulen+1);
h=x+1e-16;
a=x;
b=x;
warning ('off')
for j=2:length(xl)
    a1=(ul(j)*xl(j-1)-ul(j-1)*xl(j))/(ul(j)-ul(j-1));
    a2=(xl(j)-xl(j-1))/(ul(j)-ul(j-1));
    a3=G*(ul(j)-ul(j-1))-(xl(j)-xl(j-1));

    x((ul(j-1)*ulen+1):(ul(j)*ulen+1))=a1+a2*u((ul(j-1)*ulen+1):(ul(j)*ulen+1))+a3*sin(2*pi*(u((ul(j-1)*ulen+1):(ul(j)*ulen+1))-ul(j-1))/(ul(j)-ul(j-1)))/2/pi;
    h((ul(j-1)*ulen+1):(ul(j)*ulen+1))=a2+a3*cos(2*pi*(u((ul(j-1)*ulen+1):(ul(j)*ulen+1))-ul(j-1))/(ul(j)-ul(j-1)))/(ul(j)-ul(j-1));
end
warning ('on')
% End of defining new coordinate
% -------------------------------------------------------------------------

% ------------------------------------------
% Construction of required toeplitz matrices
% ------------------------------------------
ul(2) = ul(2)+1e-8;
hmn=ftaubu(epsl./epsl,ul,xl,G,2*N).';
Hmn=toeplitz(hmn(2*N+1:4*N+1),hmn(2*N+1:-1:1));

amn=ftaubu(epsl,ul,xl,G,2*N).';
Amn=toeplitz(amn(2*N+1:4*N+1),amn(2*N+1:-1:1));
% ------------------------------------------

% Solving eigenvalue problem in grating region
Lm=inv(Hmn)*(k0^2*Amn-Kx*inv(Hmn)*Kx);
[Ej,eigLm]=eig(Lm);

% Solving eigenvalue problem in incident region
L1=(k0^2*I)*n1^2-inv(Hmn)*Kx*inv(Hmn)*Kx;
[E1,eigL1]=eig(L1);

% Solving eigenvalue problem in transmission region
if n1==n3
    E3=E1;
    eigL3=eigL1;
else
L3=(k0^2*I)*n3^2-inv(Hmn)*Kx*inv(Hmn)*Kx;
[E3,eigL3]=eig(L3);
end

eigLm=eigLm.^.5;
eigL1=eigL1.^.5;
eigL3=eigL3.^.5;
expeigLm=zeros(2*N+1);
Y1=zeros(1,2*N+1);
Yj=Y1;
for j=1:2*N+1
    if real(eigLm(j,j))<-1e-10
        eigLm(j,j)=-eigLm(j,j);
    end
    if imag(eigLm(j,j))>1e-10
        eigLm(j,j)=-eigLm(j,j);
    end
    
    if real(eigL1(j,j))<-1e-10
        eigL1(j,j)=-eigL1(j,j);
    end
    if imag(eigL1(j,j))>1e-10
        eigL1(j,j)=-eigL1(j,j);
    end
    
    if real(eigL3(j,j))<-1e-10
        eigL3(j,j)=-eigL3(j,j);
    end
    if imag(eigL3(j,j))>1e-10
        eigL3(j,j)=-eigL3(j,j);
    end
    Y1(j)=((eigL1(j,j)));
    Yj(j)=((eigLm(j,j)));
    expeigLm(j,j)=exp(-i*eigLm(j,j)*d);
end

Qj=-i*(Ej)*eigLm;
Q1=-i*(E1)*eigL1;
Q3=-i*(E3)*eigL3;

K=zeros(2*N+1);
for p=1:2*N+1
    kp=(fastft(h.*exp(i*(kx(p)*x-kx(N+1)*u)),N)).';
    kp(1:2*N+1)=kp((2*N+1):-1:1);
    K(p,:)=kp;
end
if G==1
    K=I;
end
Wj11=Ej;
Wj12=Ej;
Wj21=Qj;
Wj22=-Qj;

W111=E1;
W112=E1;
W121=Q1;
W122=-Q1;

W311=E3;
W312=E3;
W321=Q3;
W322=-Q3;

% ------------------
% S-matrix algorithm
% ------------------
s1=inv([Wj11 -W112; Wj21 -W122])*[W111 -Wj12;W121 -Wj22 ];

Tuu=s1(1:(2*N+1),1:(2*N+1));
Rud=s1(1:(2*N+1),(2*N+2):(4*N+2));
Rdu=s1((2*N+2):(4*N+2),1:(2*N+1));
Tdd=s1((2*N+2):(4*N+2),(2*N+2):(4*N+2));


sj=[I Zero;Zero expeigLm]*inv([W311 -Wj12; W321 -Wj22])*[Wj11 -W312;Wj21 -W322 ]*[expeigLm Zero;Zero I];
tuu=sj(1:(2*N+1),1:(2*N+1));
rud=sj(1:(2*N+1),(2*N+2):(4*N+2));
rdu=sj((2*N+2):(4*N+2),1:(2*N+1));
tdd=sj((2*N+2):(4*N+2),(2*N+2):(4*N+2));
% ------------------

incident=zeros(2*N+1,1);
[amin,incI]=min(Y1-kz1(N+1));
incident(incI)=1;

Rxdu=Rdu+Tdd*rdu*inv(I-Rud*rdu)*Tuu;
Txuu=tuu*inv(I-Rud*rdu)*Tuu;
Rdu=Rxdu;Tuu=Txuu;



Tte=K*E3*Tuu*incident;
Rte=K*E1*Rdu*incident;

DE1=zeros(1,2*N+1);
DE3=zeros(1,2*N+1);
for j=1:2*N+1
    DE1(j)=(abs(Rte(j))^2).*real(kz1(j)/k1/cos(theta));
    DE3(j)=(abs(Tte(j))^2).*real(kz3(j)/(k1*cos(theta)));
end
sum(DE3)